package signtools;

import java.security.PrivateKey;
import java.security.cert.X509Certificate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

public class LaunchValidateSignedFile {
    private boolean argsError = false;
    private List listErrorArgs = new ArrayList();
    private HashSet usedFlags = new HashSet();
    private Map options = new HashMap();

    private String inputFile;
    private String outputPath;

    private ValidateSignedFile val;

    private static Logger LOGGER = Logger.getLogger("signtools.LaunchValidateSignedFile");

    /**
     * Executable method.
     * @param args: Args must be a combination of parameter specification (flag, -something) and value (without -) see 
     * initUsedArgs for a list of parameters specification (-ct FileToValidate -b OutputPath)
     */
    public static void main(String[] args) {
        LaunchValidateSignedFile vsf = new LaunchValidateSignedFile();
        if (vsf.loadArgs(args)) {
            //vsf.launch("c:/temp/signed.xml","c:/temp/target/output/");
            vsf.launch(vsf.getInputFile(), vsf.getOutputPath());
        }
    }


    /**
     * Launch signature validation of input file and extract enveloping file to outputPath.
     * @param input Input file to be validated
     * @param outputPath Path where the enveloping file will be extracted.
     */
    private void launch(String input, String outputPath) {
        try {
            setVal(new ValidateSignedFile());
            getVal().ValidateSignedFile(input, outputPath);
        } catch (Exception e) {
            LOGGER.warning(e.getMessage());
        }
    }
    
    /**
     * Load and initilize arguments.
     * @param args: Arguments, same as in main.
     * @return: true if everything is ok. false if there is any problem. (note, true/false as boolean primitive, not Boolean object).
     */
    private boolean loadArgs(String[] args) {
        this.initUsedArgs();
        boolean resFlag = false;
        parse(args);
        String resStr = checkArgs();
        if (resStr.length() == 0) {
            resFlag = true;
        } else {
            resFlag = false;
            LOGGER.warning("Syntax error/s:\n" +
                    resStr);
        }
        return resFlag;
    }
    
    /**
     * Initializes usedFlag with a list of allowed flags (parameter specifications)
     */
    private void initUsedArgs() {
        getUsedFlags().add("-ct"); //File to validate/extract signature.
        getUsedFlags().add("-b"); //Path to generate zip etc . . .
    }

    /**
     *Do whatever checking on the arguments
     * @return: String with the error/s (if any)
     */
    private String checkArgs() {
        String strRes = "";
        String param, message;
        if (getOptions() == null || getOptions().isEmpty()) {
            message = "Use with parameters: " + "-ct FileToValidate " + "-b OutputPath";
            strRes = strRes + message;
        } else {
            param = "-ct";
            message = "Missing parameter -ct. Use: -ct FileToValidate\n";
            if (!getOptions().containsKey(param) || ((String)getOptions().get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                setInputFile(((String)getOptions().get(param)).trim());
            }
            param = "-b";
            message = "Missing parameter -b. Use: -b OutputPath\n";
            if (!getOptions().containsKey(param) || ((String)getOptions().get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                setOutputPath(((String)getOptions().get(param)).trim());
            }
        }
        return strRes;
    }

    /**
     * Parse parameters from arguments to its corresponding place (inside options)
     * @param args: Same as in the main method
     * @return: true if everything is ok. false otherwise.
     */
    private boolean parse(String[] args) {
        for (int n = 0; n < args.length; ++n) {
            if (args[n].charAt(0) == '-') {
                String name = args[n];
                String value = null;

                if (getUsedFlags().contains(args[n]) && n < args.length - 1) {
                    value = args[++n];
                    getOptions().put(name, value);
                }
            }

            else {
                getListErrorArgs().add(args[n]);
                setArgsError(true);
            }
        }
        return isArgsError();
    }

    public boolean isArgsError() {
        return argsError;
    }

    public void setArgsError(boolean argsError) {
        this.argsError = argsError;
    }

    public List getListErrorArgs() {
        return listErrorArgs;
    }

    public void setListErrorArgs(List listErrorArgs) {
        this.listErrorArgs = listErrorArgs;
    }

    public HashSet getUsedFlags() {
        return usedFlags;
    }

    public void setUsedFlags(HashSet usedFlags) {
        this.usedFlags = usedFlags;
    }

    public Map getOptions() {
        return options;
    }

    public void setOptions(Map options) {
        this.options = options;
    }

    public String getInputFile() {
        return inputFile;
    }

    public void setInputFile(String inputFile) {
        this.inputFile = inputFile;
    }

    public String getOutputPath() {
        return outputPath;
    }

    public void setOutputPath(String outputPath) {
        this.outputPath = outputPath;
    }

    public ValidateSignedFile getVal() {
        return val;
    }

    public void setVal(ValidateSignedFile val) {
        this.val = val;
    }
}
